/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef COMMAND_DATA_HPP
#define COMMAND_DATA_HPP

#include <DataTypes/DxcData.hpp>
#include <DataTypes/Command.hpp>
#include <DataTypes/Value.hpp>
#include <DxcDefs.hpp>

#include <map>

namespace Dxc
{
/**
 * @brief Command datatype.
 */
class DXCAPI_EXPORT CommandData : public DxcData
{
public:

    CommandData();

    /**
     * @brief Constructor.
     *
     * @param timestamp - message timestamp
     * @param commandID - command ID string
     * @param commandValue - the Value of the command
     */
    CommandData(long long timestamp, const std::string& commandID, Value* commandValue);

    /**
     * @brief Constructor.
     *
     * @param timestamp - message timestamp
     * @param commands - map of command IDs to Values
     */
    CommandData(long long timestamp, const CommandSet& commands);

    /**
     * @brief Constructor.
     *
     * @param commands - map of command IDs to Values
     */
    CommandData(const CommandSet& commands);

    /**
     * @brief Get commands.
     */
    const CommandSet* getCommands() const;

    /**
     * @brief Set commands.
     */
    void setCommands(const CommandSet&);

    virtual std::ostream &put(std::ostream &) const;

    /**
     * @brief Get command ID string
     */
    std::string getCommandID() const;

    /**
     * @brief Get command Value
     */
    const Value* getCommandValue() const;

    virtual CommandData* clone() const;

private:
    std::ostream &putCommands(std::ostream &) const;

    CommandSet commands_;
};
}

#endif
