/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include "Fault.hpp"

using namespace std;
using namespace Dxc;

Fault::Fault(const string& componentID, const string& modeID)
    : componentID_(componentID), modeID_(modeID), parameters_()
{
}

Fault::Fault(const string& componentID, const string& modeID, const Fault::ParameterMap& parameters)
    : componentID_(componentID), modeID_(modeID), parameters_(parameters)
{
}

string Fault::getComponentID() const
{
    return componentID_;
}

string Fault::getModeID() const
{
    return modeID_;
}

const Fault::ParameterMap* Fault::getParameters() const
{
    return &parameters_;
}

void Fault::setParameters(const ParameterMap& pm)
{
    parameters_ = pm;
}

ostream &Fault::put(ostream & out) const
{
    out << componentID_ << " = " << modeID_;

    if (!parameters_.empty())
    {
        bool fg = false;
        out << " [ ";
        for (ParameterMap::const_iterator i = parameters_.begin();
                i != parameters_.end();
                i++)
        {
            if (fg)
            {
                out << ", ";
            }
            else
            {
                fg = true;
            }
            out << i->first << " = " << i->second;
        }
        out << " ]";
    }

    return out;
}

bool Fault::operator<(const Fault& f1) const
{
    if( this->getComponentID() == f1.getComponentID() &&
            this->getModeID() < f1.getModeID()) return true;
    if( this->getComponentID() < f1.getComponentID() ) return true;
    return false;
}

bool Fault::operator==(const Fault& f1) const
{
    if( this->getComponentID() != f1.getComponentID() ||
            this->getModeID() != f1.getModeID()) return false;
    return true;
}

ostream & operator << (ostream &out, const Fault& f)
{
    return f.put(out);
}

Fault::Fault(const Fault& f)
{
    componentID_ = f.getComponentID();
    modeID_ = f.getModeID();

    Fault::ParameterMap::const_iterator j = f.getParameters()->begin();

    for (; j != f.getParameters()->end(); j++)
        parameters_[j->first] = j->second->clone();
}

Fault& Fault::operator=(const Fault& f)
{
    if (this != &f)
    {
        componentID_ = f.getComponentID();
        modeID_ = f.getModeID();

        Fault::ParameterMap::iterator i = parameters_.begin();

        for (; i != parameters_.end(); i++)
            delete i->second;

        parameters_.clear();


        Fault::ParameterMap::const_iterator j = f.getParameters()->begin();

        for (; j != f.getParameters()->end(); j++)
            parameters_[j->first] = j->second->clone();
    }

    return *this;
}

Fault::~Fault()
{
    Fault::ParameterMap::iterator i = parameters_.begin();

    for (; i != parameters_.end(); i++)
        delete i->second;

    parameters_.clear();
}

ostream &FaultSet::put(ostream &out) const
{
    out << "{ ";

    if (!this->empty())
    {
        bool fg = false;
        for (FaultSet::const_iterator i = this->begin();
                i != this->end();
                i++)
        {
            if (fg)
            {
                out << ", ";
            }
            else
            {
                fg = true;
            }
            out << *i;
        }
    }

    out << " }";

    return out;
}

ostream & operator << (ostream &out, const Dxc::FaultSet& fs)
{
    fs.put(out);
    return out;
}
