/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/

#include <iostream>

#include <ScenarioTracker.hpp>
#include <DxcDefs.hpp>

#ifndef _WIN32
#include <sys/time.h>
#endif
#include <time.h>
#include <errno.h>
#include <sys/timeb.h>

using namespace Dxc;
using namespace std;

ScenarioTracker::ScenarioTracker()
{
    status_ = ST_INIT;
    pthread_cond_init(&statUpdated_, NULL);
    pthread_mutex_init(&condMutex_, NULL);
    pthread_mutex_init(&statMutex_, NULL);
}

int ScenarioTracker::getStatus()
{
    return status_;
}

void ScenarioTracker::updateStatus(int newStatus)
{
    pthread_mutex_lock(&statMutex_);
    status_ = newStatus;
    pthread_mutex_unlock(&statMutex_);
    pthread_cond_broadcast(&statUpdated_);
}

int ScenarioTracker::waitForUpdate(long timeoutMillis)
{
    struct timeval systemTime;
    timespec t;

    gettimeofday(&systemTime, NULL);

    t.tv_sec = systemTime.tv_sec + (timeoutMillis / 1000);
    t.tv_nsec = (systemTime.tv_usec / 1000 + (timeoutMillis % 1000)) * 1000000;

    pthread_mutex_lock(&condMutex_);
    int waitval = pthread_cond_timedwait(&statUpdated_, &condMutex_, &t);
    pthread_mutex_unlock(&condMutex_);

    if (waitval == ETIMEDOUT)
    {
        return ST_WAIT_TIMEOUT;
    }
    else
    {
        return status_;
    }
}

int ScenarioTracker::waitForUpdate() // no timeout
{
    pthread_mutex_lock(&condMutex_);

    int waitval = pthread_cond_wait(&statUpdated_, &condMutex_);

    if (waitval == ETIMEDOUT)
    {
        return ST_WAIT_TIMEOUT;
    }
    else
    {
        return status_;
    }
}
