/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifdef _WIN32
#include <Windows.h>
#else
#include <unistd.h>
#include <stdlib.h>
#endif

#include <iostream>
#include <fstream>

#include <DxcCallback.hpp>
#include <DataTypes/DxcData.hpp>
#include <Connection/ConnectorFactory.hpp>

using namespace Dxc;
using namespace std;

static DxcConnector *connector;
static ofstream *outFile;
static string daName = "";
static string daVersion = "";
static bool run = true;

class RecordCallback : public DxcCallback
{
public:
    virtual void processData(const DxcData *msg)
    {
        DxcData* d = msg->clone();
        d->setTimeStamp(DxcData::currentTime());

        if (string(typeid(*d).name()) == string(typeid(ScenarioStatusData).name()))
        {
            const ScenarioStatusData* stat = dynamic_cast<const ScenarioStatusData *>(d);

            string status = stat->getStatus();

            if (status == ScenarioStatusData::DA_ENDED)
            {
                run = false;
            }
        }
        else if (string(typeid(*d).name()) == string(typeid(ScenarioData).name()))
        {
            ScenarioData *sd = dynamic_cast<ScenarioData*>(d);
            sd->setDAName(daName);
            sd->setDAVersion(daVersion);
            sd->setFrameworkVersion("DXC10");
        }
        else if (string(typeid(*d).name()) == string(typeid(ErrorData).name()))
        {
            run = false;
        }

        if (string(typeid(*d).name()) != string(typeid(SensorData).name()))
        {
            try
            {
                connector->sendMessage(d);
            }
            catch (...)
                {}
        }

        if (outFile && outFile->is_open())
        {
            *outFile << d;
        }

        delete d;
    };
};

void removeQuotes(string& str)
{
    size_t n = str.find_first_of("'\"");
    while (n != string::npos)
    {
        str.erase(n, 1);
        n = str.find_first_of("'\"", n);
    }
}

int main(int argc, char **argv)
{
    if (argc != 4)
    {
        cerr << (string)argv[0] + ": DAName, version, and name of " <<
             "output file must be passed in command line. " << endl;
        exit(1);
    }
    else
    {
        string s(argv[3]);
        daName = argv[1];
        daVersion = argv[2];
        removeQuotes(s);
        outFile = new ofstream(s.c_str());
    }

    RecordCallback callback;

    try
    {
        connector = ConnectorFactory::getScenarioRecorderConnector(&callback);
    }
    catch (...)
    {
        cerr << "ScenarioRecorder unable to connect. " <<
             "Make sure ScenarioLoader is running." << endl;
        return 1;
    }

    ScenarioStatusData ssd(ScenarioStatusData::SR_READY);
    if (connector)
    {
        connector->sendMessage(&ssd);
    }

    while (run)
    {
        msleep(500);
    }


    delete connector;
    connector = NULL;

    return 0;
}
